package oco;

import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Properties;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import oco.erreur.InvalidXmlException;
import oco.erreur.OutMessage;
import oco.erreur.RessourceException;
import oco.erreur.MessageErrorManager;
import oco.regles.FactoryRules;
import oco.regles.Regle;
import oco.structure.ElementStructure;
import oco.structure.FactoryElementStructurel;
import oco.traitement.CoNetcdfFile;
import oco.traitement.ConstantesProperties;
import oco.traitement.OperationFichier;
import oco.traitement.XmlManager;

import ucar.nc2.NetcdfFile;

public class FormatControl {

	static public MessageErrorManager report = null;
	
	
	public static void main(String[] args) {

		// Redefinition de la sortie des erreurs pour le parser xerces : ne surtout pas faire ca !
		// A faire : catcher les erreurs
//		PipedOutputStream fos = new PipedOutputStream();
//		PrintStream ps = new PrintStream(fos);
//		System.setErr(ps);

		report = new MessageErrorManager();
		report.begin();
		
		// Chargement des properties de l'application
		Properties appProperties = null;
		String fileName = "";
		try {
			fileName = System.getProperty(ConstantesProperties.applicationProperties);
			appProperties = OperationFichier.chargerFichierProperties(fileName);
		} catch (RessourceException e) {
			report.addAppError("Can't load the properties file : " + fileName);
			return;
		}
		
		// Chargement des messages d'erreurs de l'application
		Properties appMessages = null;
		try {
			fileName = (String)appProperties.get(ConstantesProperties.appErrorProperties);
			appMessages = OperationFichier.chargerFichierProperties(fileName);
		} catch (RessourceException e) {
			report.addAppError("Can't load the properties file : " + fileName);
			return;
		}
		
		// Chargement des messages d'erreurs du fichier netcdf
		Properties fileMessages = null;
		try {
			fileName = (String)appProperties.get(ConstantesProperties.fileErrorProperties);
			fileMessages = OperationFichier.chargerFichierProperties(fileName);
		} catch (RessourceException exc) {
			report.addAppError("Can't load the properties file : " + fileName);
			return;
		}
		
		// Chargement du gestionnaire des messages d'erreurs
		try {		
			report = new MessageErrorManager(appMessages, fileMessages);
			report.begin();
		} catch (Exception e) {
			report.addAppError(e.getMessage());
			return;
		}

		// Verifier le parametre
		if (args.length < 1) {
			report.addAppError(ConstantesProperties.error1);
			return;
		}
		// parametre optionel du traitement
		if (args[0].equals("applicationVersion")) {
			String version = null;
			version = (String)appProperties.get(ConstantesProperties.applicationVersion);
			report.addValue(OutMessage.version, version);
			report.end(true);
			return;
		}

		
		// Chargement du fichier netcdf
		NetcdfFile netcdfFile = null;
		String netcdfFileName = args[0];
		try {
			netcdfFile = NetcdfFile.open(netcdfFileName, null);
			report.addValue(OutMessage.netcdfFile, netcdfFileName);
		} catch (IOException e) {
			report.addAppError(ConstantesProperties.error2, netcdfFileName);
			return;
		}
		CoNetcdfFile fichierNetcdf = new CoNetcdfFile(netcdfFile);
		

		// Lister les fichiers de regles
		ArrayList<File> rulesFileList = null;
		String rulesDirectory = null;
		try {
			rulesDirectory = (String)appProperties.get(ConstantesProperties.rulesDirectory);
			rulesFileList = OperationFichier.recupererDesFichiers(rulesDirectory, ".xml");

			if (rulesFileList ==  null || rulesFileList.isEmpty() == true) {
				FormatControl.report.addAppError(ConstantesProperties.error3);
				throw new RessourceException();
			}

			// Test : Forcer le choix du fichier de rgles
			if (args.length != 1){
				String rulesFileName = args[1];
				File rulesFile = null;
				rulesFileList.clear();
				rulesFile = new File(rulesFileName);
				if (rulesFile.exists() == true) {
					rulesFileList.add(rulesFile);
				}
				else {
					report.addValue("test", "The rules file doesn't exist");
				}
			}
		} catch (RessourceException e) {
			return;
		}


		// Chargement du fichier xsd de validation
		String xsdDirectory = null;
		File xsdFile = null;
		String xsdFileName = null;
		try {
			xsdFileName = (String)appProperties.get(ConstantesProperties.xsdFile);
			xsdDirectory = (String)appProperties.get(ConstantesProperties.rulesDirectory);
			xsdFile = OperationFichier.recupererUnFichier(xsdDirectory, xsdFileName);
			if (xsdFile == null) {
				report.addAppError(ConstantesProperties.error4, xsdFileName);
				throw new RessourceException();
			}
		} catch (RessourceException e) {
			return;
		}
		

		XmlManager xmlManager = null;
		Regle rule = null;
		ElementStructure elemStruct = null;
		Document doc = null;
		boolean bFin = false;

		// pour chaque fichier de regles
		Iterator iterRulesFileList = rulesFileList.iterator();
		while (iterRulesFileList.hasNext() && bFin == false) {
			File rulesFile = (File)iterRulesFileList.next();
        	// Chargement du fichier de rgles
	        try {
	        	xmlManager = new XmlManager(rulesFile, report);
	        } catch (RessourceException e) {
				continue;
			}catch (InvalidXmlException e) {	
        		continue;
        	}
			

        	// Le fichier de regles est il valide par rapport au xsd ?
	        try {
	        	xmlManager.verifierFichierRegle(xsdFile);
	        } catch (RessourceException e) {
				continue;
			} catch (InvalidXmlException e) {
				// Erreur de syntaxe par rapport au xsd
				continue;
			}
			

        	// Le fichier de regles est il applicable ?
	        try {
	        	xmlManager.verifierApplicabilite(fichierNetcdf);
	        } catch (RessourceException ressource) {
	        	// Le fichier n'est pas applicable 
	        	continue;
	        }
	        
	        
	        // Le fichier est applicable
			bFin = true;
	        doc = xmlManager.getDoc();
			NodeList lstXmlRule = doc.getDocumentElement().getChildNodes();
			
			// Pour chaque regle
			for (int i = 0; i < lstXmlRule.getLength(); i++) {
				Node xmlRule = lstXmlRule.item(i);
				if (xmlRule.getNodeType() == Node.ELEMENT_NODE) {
					
					// creation de la regle
					rule = FactoryRules.createRule(xmlRule, report, fichierNetcdf);
					if (rule == null) {
						continue;
					}
					
					// Pour chaque element
					NodeList lstXmlElemStruct = xmlRule.getChildNodes();
					for (int j = 0;j<lstXmlElemStruct.getLength(); j++) {
						Node xmlElemStruct = lstXmlElemStruct.item(j);
						if (xmlElemStruct.getNodeType() == Node.ELEMENT_NODE && rule != null) {
							elemStruct = FactoryElementStructurel.createElementStructurel(xmlElemStruct);
							if (elemStruct == null) {
								continue;
							}
							rule.add(elemStruct);
						}
					}
					
					// application de la regle
					rule.executerRegles();
				}
			}
		}

		if (bFin == false) {
			report.addAppError(ConstantesProperties.error5);
			return;
		}

		// Affichage du rapport d'execution
		report.end();
		
	}
	
	
	
	/**
	 * pour les tests unitaires
	 * 
	 */
	public static MessageErrorManager executerTest(String netcdfFile, String rulesFile) 
    {
		String[] arguments = {netcdfFile, rulesFile};
		main(arguments);		
		return report;
	}
}
