package oco.erreur;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.TimeZone;

import oco.traitement.ConstantesXml;

public class MessageErrorManager {

    StringBuffer stringReport = null;
    
    Properties appMessages = null;
    boolean status = true;
    
    Properties fileMessages = null;
    boolean compliant = true;

    
	/**
	 * Constructeur
	 * 
	 */
    public MessageErrorManager() {
    	this.stringReport = new StringBuffer();
    	this.appMessages = null;
    	this.fileMessages = null;
    }
    
	/**
	 * Constructeur
	 * 
	 */
    public MessageErrorManager(Properties fileMessages) {
    	this.stringReport = new StringBuffer();
    	this.appMessages = null;
    	this.fileMessages = fileMessages;
    }
    
	/**
	 * Constructeur
	 * 
	 */
    public MessageErrorManager(Properties appMessages, Properties fileMessages) {
    	this.stringReport = new StringBuffer();
    	this.appMessages = appMessages;
    	this.fileMessages = fileMessages;
    }

    
	/**
	 * Initialise le rapport
	 * 
	 */
    public void begin() {
    	this.stringReport.append(OutMessage.xmlHeader);
    	this.stringReport.append(OutMessage.reportOpenTag);
    	this.addValue(OutMessage.function, "CO-03-08-03");
    	this.addValue(OutMessage.comment, "Control file data format");
        SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
        dateFormat.setTimeZone(TimeZone.getTimeZone("Universal"));
        this.addValue(OutMessage.date, dateFormat.format(new Date()));
    }
    
	/**
	 *  Erreur du traitement
	 * 
	 */
    public void addAppError(String error, String infoSupp) {
    	String message;
    	if (appMessages != null && appMessages.containsKey(error))
    	{
    		message = (String)appMessages.getProperty(error);
    	}
    	else {
        	message = error;
    	}
        message = message + infoSupp;
		this.addValue(OutMessage.error, message);
        this.status = false;
        this.compliant = false;
		this.end(false);
		return;
	}
    
    public void addAppError(String messageNumber) {
    	this.addAppError(messageNumber , "");
	}

	/**
	 * Erreur sur le fichier de regle
	 * 
	 */
    public void addRuleError(String error, String infoSupp) {
    	String message;
    	if (appMessages != null && appMessages.containsKey(error))
    	{
    		message = (String)appMessages.getProperty(error);
    	}
    	else {
        	message = error;
    	}
        message = message + infoSupp;
		this.addValue(OutMessage.rule_error, message);
		return;
	}
    
    public void addRuleError(String messageNumber) {
    	this.addRuleError(messageNumber , "");
	}

    
	/**
	 * Erreur sur le fichier netcdf
	 * 
	 */
    public void addFileError(String messageNumber, HashMap datas, String errorLevel) {

    	String message = formatMessage((String)fileMessages.getProperty(messageNumber), datas);

    	if (errorLevel == null) {
    		this.addValue(OutMessage.file_error, message);
    		this.compliant = false;
    	}
    	else if (errorLevel.equals(ConstantesXml.ERROR)) {
    		this.addValue(OutMessage.file_error, message);
    		this.compliant = false;
    	}
    	else if (errorLevel.equals(ConstantesXml.WARNING)) {
    		this.addValue(OutMessage.file_warning, message);
    	}
    }
    

	/**
	 * Ferme et affiche le rapport
	 * 
	 */
    public void end(boolean p_status) {
		this.status = p_status;
		this.end();
    }
    
    public void end() {
		// COMPLIANT
		if (this.status == true) {
			if (this.compliant == true) {
				this.addValue(OutMessage.compliant, "yes");
			}	
			else {
				this.addValue(OutMessage.compliant, "no");
			}
		}
		// STATUS
		if (this.status == true) {
			this.addValue(OutMessage.status, "ok");
		}
		else {
			this.addValue(OutMessage.status, "nok");
		}
		
		this.stringReport.append(OutMessage.reportCloseTag);
        System.out.println(this.stringReport.toString());  
    }

	/**
	 * ajoute une ligne dans le rapport 
	 * 
	 */
    public void addValue(String tag, String value) {
        HashMap<String,String> keyAndDatas = new HashMap<String,String>();
        keyAndDatas.put("{0}", tag);
        keyAndDatas.put("{1}", value);
        String message = formatMessage(OutMessage.pattern, keyAndDatas);
        this.stringReport.append(message);
    }
    
	/**
	 * remplace chaque {i} par une info
	 * 
	 */
	public static String formatMessage(String message, HashMap dynamicData) {
		
		if ((message != null) && (message.length() != 0)) {
			
			for (int i = 0; i < dynamicData.size(); i++) {
				
				StringBuffer temp = new StringBuffer();
				temp.append("{").append(i).append("}");
				String valeur = (String)dynamicData.get(temp.toString());
				if (valeur != null) {
					if (valeur.matches(".*\\.*")) {
						valeur = valeur.replaceAll("\\\\"," ");
					}
					if (valeur.matches(".*$.*")) {
						valeur = valeur.replaceAll("\\$"," ");
					}
					message = message.replaceAll("\\" + temp.toString(), valeur);
				}
			}
		}
		else {
			return "";
		}
		return message;
	}
    
	/**
	 * pour les tests unitaires
	 * 
	 */
    public boolean isCompliant() {

		return this.compliant;
    }
}
