package oco.regles;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oco.erreur.ErreurRegles;
import oco.erreur.MessageErrorManager;
import oco.structure.CoVariable;
import oco.structure.ElementStructure;
import oco.traitement.CoNetcdfFile;
import oco.traitement.ConstantesProperties;
import oco.traitement.ConstantesXml;

public class AtLeast extends Regle {

    /**
     * Constructeur 
     * 
     */
	public AtLeast(CoNetcdfFile p_file, MessageErrorManager p_report, String p_errorLevel) {
		super(p_file, p_report, p_errorLevel);
		// Appliquable a VARIABLE
		netcdfDatas.put(ConstantesXml.VARIABLE, this.netcdfFile.retournerVariables());
	}

    /**
     *  executerRegles
     * 
     */
	public void executerRegles() {
		boolean bAtLeast = false;
		CoVariable ruleVariable = new CoVariable();
		ArrayList<CoVariable> variableList = new ArrayList<CoVariable>();

		// Pour chaque element at_least du fichier de regle
		Iterator iterElem = ruleElements.iterator();
		while (iterElem.hasNext()) {
			ElementStructure elemRegle = (ElementStructure)iterElem.next();
	        elemRegle.elementBlocLevel = this.ruleBlocLevel;
	        
			// VARIABLE 
			if (elemRegle instanceof CoVariable) {
				
	        	// Recuperer l'element a verifier
				ruleVariable = (CoVariable)elemRegle;
				
				// Stocker l'element pour le message d'erreur
				variableList.add(ruleVariable);
				
				//	Recuperer les donnees netcdf correspondantes
				List netcdfList = (List) netcdfDatas.get(elemRegle.toString());
				
	            // Gerer le niveau de l'erreur : ERROR ou WARNING
	            String errorLevel = null;
	            if (elemRegle.elementErrorLevel != null) {
	            	errorLevel = elemRegle.elementErrorLevel;
	            }
	            else {
	            	if (this.ruleErrorLevel != null) {
	            		errorLevel = this.ruleErrorLevel;
	            	}
	            }
				
	            // Verifier la variable si elle est prsente
				if (ruleVariable.variableExiste(netcdfList) == true) {
					bAtLeast = true;
					gererErreur(ruleVariable.verifierVariable(), errorLevel);
				}
			}
		}
		
		// Aucune variable at_least n'est presente dans le fichier netcdf
		if (bAtLeast == false) {
			
            // Gerer le niveau de l'erreur : ERROR ou WARNING
            String errorLevel = null;
            if (this.ruleErrorLevel != null) {
            	errorLevel = this.ruleErrorLevel;
            }
            
			ArrayList<String []> lstErreur = new ArrayList<String []>();
			String [] tab = {String.valueOf(ErreurRegles.VAR_NOT_FOUND), this.recupElemList(variableList)};
			lstErreur.add(tab);
			gererErreur(lstErreur, errorLevel);
		}
	}
	
    /**
     * 
     * 
     */
	private String recupElemList(ArrayList variableList) {
		StringBuffer lstNom = new StringBuffer();
		
		Iterator iter = variableList.iterator();
		while (iter.hasNext()) {
			ElementStructure elem = (ElementStructure)iter.next();
			if (elem.identifyBy != null) {
				lstNom.append(elem.identifyBy);
			}
			if (iter.hasNext()) {
				lstNom.append(", ");
			}
		}
		return lstNom.toString();
	}
	
    /**
     *  recupererMessageErreur
     * 
     */
	protected String recupererMessageErreur(int codeErreur) {
		String messProperties = null;

        switch (codeErreur) {
            case ErreurRegles.VAR_NOT_FOUND :
            	messProperties = ConstantesProperties.atLeastRule1;
            	break;
            case ErreurRegles.ATT_VAR :
            	messProperties = ConstantesProperties.atLeastRule4;
            	break;
            case ErreurRegles.ATT_VAR_VALUE :
            	messProperties = ConstantesProperties.atLeastRule7;
            	break;
            case ErreurRegles.DIM_MUTEX_VAR :
            	messProperties = ConstantesProperties.atLeastRule5;
            	break;
            case ErreurRegles.DIM_MUTEX_VAR_NOT_FOUND :
            	messProperties = ConstantesProperties.atLeastRule9;
            	break;
            case ErreurRegles.DIM_VAR :
            	messProperties = ConstantesProperties.atLeastRule3;
            	break;
            case ErreurRegles.NB_DIM_VAR :
            	messProperties = ConstantesProperties.atLeastRule8;
            	break;
            case ErreurRegles.TYPE_MUTEX_VAR :
            	messProperties = ConstantesProperties.atLeastRule6;
            	break;
            case ErreurRegles.TYPE_VAR :
            	messProperties = ConstantesProperties.atLeastRule2;
            	break;
            case ErreurRegles.TYPE_MUTEX_VAR_NOT_FOUND :
            	messProperties = ConstantesProperties.atLeastRule10;
            	break;
            default :
            	break;
        }
        return messProperties;
	}
}
