package oco.regles;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oco.erreur.ErreurRegles;
import oco.erreur.MessageErrorManager;
import oco.structure.ElementStructure;
import oco.traitement.CoNetcdfFile;
import oco.traitement.ConstantesProperties;
import oco.traitement.ConstantesXml;

public class Mandatory extends Regle {

    /**
     * Constructeur 
     * 
     */
    public Mandatory(CoNetcdfFile p_file, MessageErrorManager p_report, String p_errorLevel) {
        super(p_file, p_report, p_errorLevel);
        // Regle susceptible de s'appliquer a tous
        netcdfDatas.put(ConstantesXml.DIMENSION, this.netcdfFile.retournerDimensions());
        netcdfDatas.put(ConstantesXml.VARIABLE, this.netcdfFile.retournerVariables());
        netcdfDatas.put(ConstantesXml.ATTRIBUTE, this.netcdfFile.retournerGlobalAttributes());
    }

    /**
     * Constructeur pour les tests
     * 
     */
    public Mandatory(CoNetcdfFile p_file, List [] datas, MessageErrorManager p_report, String p_errorLevel) {
        super(p_file, p_report, p_errorLevel);
        // Regle susceptible de s'appliquer a tous
        netcdfDatas.put(ConstantesXml.DIMENSION, datas[0]);
        netcdfDatas.put(ConstantesXml.ATTRIBUTE, datas[1]);
        netcdfDatas.put(ConstantesXml.VARIABLE, datas[2]);
    }

    /**
     *  executerRegles
     * 
     */
    public void executerRegles() {
    	
        Iterator iterElem = ruleElements.iterator();
        while (iterElem.hasNext()) {
        	
        	// Recuperer l'element a verifier
            ElementStructure elemRegle = (ElementStructure)iterElem.next();
            
            // Gerer le bloc de l'element : MANDATOTY, OPTIONAL, ...
            elemRegle.elementBlocLevel = this.ruleBlocLevel;
            
            // Recuperer les donnees netcdf correspondantes
            ArrayList netcdfList = (ArrayList)netcdfDatas.get(elemRegle.toString());
            
            // Verifier la regle
            ArrayList lstErreur = elemRegle.trouver(netcdfList, this.ruleBlocLevel);
            
            // Gerer le niveau de l'erreur : ERROR ou WARNING
            String errorLevel = null;
            if (elemRegle.elementErrorLevel != null) {
            	errorLevel = elemRegle.elementErrorLevel;
            }
            else {
            	if (this.ruleErrorLevel != null) {
            		errorLevel = this.ruleErrorLevel;
            	}
            }
            
            // Gerer le message d'ereur
            gererErreur(lstErreur, errorLevel);
        }
    }
    
    /**
     *  recupererMessageErreur
     * 
     */
    protected String recupererMessageErreur(int codeErreur) {
    	String messProperties = null;

        switch (codeErreur) {
            case ErreurRegles.DIM_NOT_FOUND :
            	messProperties = ConstantesProperties.mandatoryRule1;
                break;
            case ErreurRegles.DIM_MUTEX :
            	messProperties = ConstantesProperties.mandatoryRule2;
                break;
            case ErreurRegles.DIM_MUTEX_NOT_FOUND :
            	messProperties = ConstantesProperties.mandatoryRule14;
                break;
            case ErreurRegles.ATT_NOT_FOUND :
            	messProperties = ConstantesProperties.mandatoryRule3;
                break;
            case ErreurRegles.ATT_VALUE :
            	messProperties = ConstantesProperties.mandatoryRule4;
                break;
            case ErreurRegles.VAR_NOT_FOUND :
            	messProperties = ConstantesProperties.mandatoryRule5;
                break;
            case ErreurRegles.VAR_MUTEX_NOT_FOUND :
            	messProperties = ConstantesProperties.mandatoryRule15;
                break;
            case ErreurRegles.ATT_VAR :
            	messProperties = ConstantesProperties.mandatoryRule6;
            	break;
            case ErreurRegles.TYPE_VAR :
            	messProperties = ConstantesProperties.mandatoryRule7;
                break;
            case ErreurRegles.TYPE_MUTEX_VAR :
            	messProperties = ConstantesProperties.mandatoryRule8;
                break;
            case ErreurRegles.DIM_VAR :
            	messProperties = ConstantesProperties.mandatoryRule9;
                break;
            case ErreurRegles.DIM_MUTEX_VAR :
            	messProperties = ConstantesProperties.mandatoryRule10;
                break;
            case ErreurRegles.VAR_MUTEX :
            	messProperties = ConstantesProperties.mandatoryRule12;
                break;
            case ErreurRegles.ATT_VAR_VALUE :
            	messProperties = ConstantesProperties.mandatoryRule11;
                break;
            case ErreurRegles.NB_DIM_VAR :
            	messProperties = ConstantesProperties.mandatoryRule13;
                break;
            case ErreurRegles.DIM_MUTEX_VAR_NOT_FOUND :
                messProperties = ConstantesProperties.mandatoryRule16;
                break;
            case ErreurRegles.TYPE_MUTEX_VAR_NOT_FOUND :
            	messProperties = ConstantesProperties.mandatoryRule17;
                break;
            case ErreurRegles.DIM_VALUE :
            	messProperties = ConstantesProperties.mandatoryRule18;
                break;
            default :

            	break;
        }
        return messProperties;
    }

 }