package oco.regles;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oco.erreur.ErreurRegles;
import oco.erreur.MessageErrorManager;
import oco.structure.CoDimension;
import oco.structure.CoVariable;
import oco.structure.ElementStructure;
import oco.traitement.CoNetcdfFile;
import oco.traitement.ConstantesProperties;
import oco.traitement.ConstantesXml;

public class Optional extends Regle {

    /**
     * Constructeur 
     * 
     */
	public Optional(CoNetcdfFile p_file, MessageErrorManager p_report, String p_errorLevel) {
		super(p_file, p_report, p_errorLevel);
		// Appliquable au DIMENSION et VARIABLE
        netcdfDatas.put(ConstantesXml.DIMENSION, this.netcdfFile.retournerDimensions());
		netcdfDatas.put(ConstantesXml.VARIABLE, this.netcdfFile.retournerVariables());
	}

    /**
     *  executerRegles
     * 
     */
	public void executerRegles() {
		ArrayList lstErreur = new ArrayList();
		
		Iterator iterElem = ruleElements.iterator();
		while (iterElem.hasNext()) {
			
			// Recuperer l'element a verifier
			ElementStructure elemRegle = (ElementStructure)iterElem.next();
			
            // Gerer le bloc de l'element : MANDATOTY, OPTIONAL, ...
            elemRegle.elementBlocLevel = this.ruleBlocLevel;
            
            // Gerer le niveau de l'erreur : ERROR ou WARNING
            String errorLevel = null;
            if (elemRegle.elementErrorLevel != null) {
            	errorLevel = elemRegle.elementErrorLevel;
            }
            else {
            	if (this.ruleErrorLevel != null) {
            		errorLevel = this.ruleErrorLevel;
            	}
            }

			// DIMENSION
			if (elemRegle instanceof CoDimension) {
				CoDimension ruleDimension = (CoDimension)elemRegle;
				// Recuperer les donnees netcdf correspondantes
				ArrayList netcdfList = (ArrayList) netcdfDatas.get(elemRegle.toString());
				// Verifier la dimension si elle est prsente
				lstErreur = ruleDimension.trouver(netcdfList, this.ruleBlocLevel);
				gererErreur(lstErreur, errorLevel);
			}
			
			// VARIABLE
			if (elemRegle instanceof CoVariable) {
				CoVariable ruleVariable = (CoVariable)elemRegle;
				// Recuperer les donnees netcdf correspondantes
				List netcdfList = (List) netcdfDatas.get(elemRegle.toString());
				// Verifier la variable si elle est prsente
				if (ruleVariable.variableExiste(netcdfList) == true) {
					lstErreur = ruleVariable.verifierVariable();
					gererErreur(lstErreur, errorLevel);
				}
			}
		}
	}

    /**
     *  recupererMessageErreur
     * 
     */
	protected String recupererMessageErreur(int codeErreur) {
		String messProperties = null;

        switch (codeErreur) {
            case ErreurRegles.ATT_VAR :
            	messProperties = ConstantesProperties.optionalRule3;
            	break;
            case ErreurRegles.ATT_VAR_VALUE :
            	messProperties = ConstantesProperties.optionalRule6;
            	break;
            case ErreurRegles.DIM_MUTEX_VAR :
            	messProperties = ConstantesProperties.optionalRule4;
            	break;
            case ErreurRegles.DIM_MUTEX_VAR_NOT_FOUND :
            	messProperties = ConstantesProperties.optionalRule8;
            	break;
            case ErreurRegles.DIM_VAR :
            	messProperties = ConstantesProperties.optionalRule2;
            	break;
            case ErreurRegles.NB_DIM_VAR :
            	messProperties = ConstantesProperties.optionalRule7;
            	break;
            case ErreurRegles.TYPE_MUTEX_VAR :
            	messProperties = ConstantesProperties.optionalRule5;
            	break;
            case ErreurRegles.TYPE_VAR :
            	messProperties = ConstantesProperties.optionalRule1;
            	break;
            case ErreurRegles.TYPE_MUTEX_VAR_NOT_FOUND :
            	messProperties = ConstantesProperties.optionalRule9;
            	break;
            case ErreurRegles.DIM_VALUE :
            	messProperties = ConstantesProperties.optionalRule10;
                break;
            default :
            	break;
        }
        return messProperties;
	}
}
