package oco.structure;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import oco.erreur.ErreurRegles;
import oco.traitement.ConstantesXml;

public class Critere extends ElementCompose {

	protected Integer dimensionCount = null;

	private ArrayList<CoVariable> foundVariables = null;
	
	/**
	 * Constructeur
	 * 
	 */
	public Critere() {
		super();
		this.foundVariables = new ArrayList<CoVariable>();
	}

	/**
	 * Accesseurs
	 * 
	 */
	public ArrayList getFoundVariables() {
		return this.foundVariables;
	}
	
	public String getType() {
		return this.type;
	}
	
	public void setType(String p_type) {
		this.type = p_type;
	}

	public void setDimensionCount(String p_dimensionCount) {
		this.dimensionCount = new Integer(p_dimensionCount);
	}

	public ArrayList getDimensions() {
		return (ArrayList)this.elementList.get(ConstantesXml.DIMENSION);
	}

	public ArrayList getAttributs() {
		return (ArrayList)this.elementList.get(ConstantesXml.ATTRIBUTE);
	}

	public ArrayList getMutex() {
		return (ArrayList)this.elementList.get(ConstantesXml.MUTEX);
	}


	
	public void ajouterDimension(CoDimension dim) {
		ArrayList<ElementStructure> list = this.elementList.get(ConstantesXml.DIMENSION);
		if (list == null){
			list = new ArrayList<ElementStructure>();
		}
		list.add(dim);
		elementList.put(ConstantesXml.DIMENSION, list);
	}

	public void ajouterAttribut(CoAttribute attribute) {
		ArrayList<ElementStructure> list = this.elementList.get(ConstantesXml.ATTRIBUTE);
		if (list == null){
			list = new ArrayList<ElementStructure>();
		}
		list.add(attribute);
		this.elementList.put(ConstantesXml.ATTRIBUTE, list);	
	}

	public void ajouterMutex(Mutex mutex) {
		ArrayList<ElementStructure> list = this.elementList.get(ConstantesXml.MUTEX);
		if (list == null){
			list = new ArrayList<ElementStructure>();
		}
		list.add(mutex);
		this.elementList.put(ConstantesXml.MUTEX, list);
	}

	/**
	 * trouver
	 * trouver une variable par ses critere IDENTIFY
	 */
	public ArrayList trouver(ArrayList netcdfList, String ruleLevel) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();
		CoVariable variable = null;
		boolean exists = false;
		
		// Pour chaque variable netcdf
		Iterator iterVar = netcdfList.iterator();
		while (iterVar.hasNext()) {
			variable = (CoVariable) iterVar.next();

			if (trouverNom(variable).isEmpty() == false) {
				continue;
			}
			if (trouverPattern(variable).isEmpty() == false) {
				continue;
			}
			if (trouverDimensions(variable).isEmpty() == false) {
				continue;
			}
			if (trouverAttribut(variable).isEmpty() == false) {
				continue;
			}
			if (trouverMutex(variable).isEmpty() == false) {
				continue;
			}
			if (trouverNbDimension(variable).isEmpty() == false) {
				continue;
			}
			if (trouverType(variable).isEmpty() == false) {
				continue;
			}
			exists = true;
			foundVariables.add(variable);
		}

		// Si la variable du fichier de rgle n'est pas dans le fichier netcdf
		if (exists == false) {
			String [] tab = {String.valueOf(ErreurRegles.VAR_NOT_FOUND), 
					this.identifyBy};
			lstErreur.add(tab);
		}
		return lstErreur;
	}

	/**
	 * 
	 * verifier la/les variables trouvees par les critere MUST_VERIFY
	 */
	public void checkFoundVariables(ArrayList<String []> lstErreur, ArrayList foundVariables) {
		CoVariable variable = null;

		Iterator iterVar = foundVariables.iterator();
		while (iterVar.hasNext()) {
			variable = (CoVariable) iterVar.next();
			lstErreur.addAll(this.trouverNom(variable));
			lstErreur.addAll(this.trouverDimensions(variable));
			lstErreur.addAll(this.trouverAttribut(variable));
			lstErreur.addAll(this.trouverMutex(variable));
			lstErreur.addAll(this.trouverNbDimension(variable));
			lstErreur.addAll(this.trouverType(variable));
		}
	}

	/**
	 * 
	 * 
	 */
	private ArrayList<String []> trouverNom(CoVariable variable) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();

		if (this.getName() != null) {

			if (variable.getName().equals(this.getName()) == false) {
				
				String [] tab = {
						String.valueOf(ErreurRegles.NOM_VAR), this.getName()};
				lstErreur.add(tab);
			}
		}
		return lstErreur;
	}
	
	/**
	 * 
	 * 
	 */
	private ArrayList trouverPattern(CoVariable variable) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();
		
		if (this.getPattern() != null) {
			
			String stringPattern = this.getPattern();
			Pattern pattern = Pattern.compile(stringPattern);
			Matcher match = pattern.matcher(variable.getName());
			if (match.matches() == false) {
				String [] tab = {
						String.valueOf(ErreurRegles.NOM_VAR), 
						this.getPattern()};
				lstErreur.add(tab);
			}
		}
		return lstErreur;
	}
	
	/**
	 * 
	 * 
	 */
	private ArrayList<String []> trouverDimensions(CoVariable variable) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();
		ArrayList<ElementStructure> dimensionList = null;
		String dimensionName = "";
		String dimensionRank = "";

		if ((this.getDimensions() != null) && (this.getDimensions().isEmpty() == false)) {

			Iterator iterDim = this.getDimensions().iterator();
			while (iterDim.hasNext()) {
				
				CoDimension ruleDimension = (CoDimension) iterDim.next();
				dimensionList = variable.getDimensions();
				if (ruleDimension.trouver(dimensionList, this.elementBlocLevel).isEmpty() == false ) {
					dimensionName = ruleDimension.getName();
					if (ruleDimension.getRank() != null) {
						dimensionRank = ruleDimension.getRank().toString();
					}
					String [] tab = {
							String.valueOf(ErreurRegles.DIM_VAR),
							variable.getName(),
							dimensionName,
							dimensionRank
						};
					lstErreur.add(tab);
				}
			}
		}
		return lstErreur;
	}

	/**
	 * 
	 * 
	 */
	private ArrayList<String []> trouverMutex(CoVariable var) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();
		
		if (this.getMutex() != null && this.getMutex().isEmpty() == false) {

			Iterator iterMutex = this.getMutex().iterator();
			while (iterMutex.hasNext()) {
				
				ArrayList<ElementStructure> lstElemMutex = null;
				ArrayList lstElemErreur = null;
				Mutex mutex = (Mutex) iterMutex.next();

				if (mutex.toString().equals(ConstantesXml.DIMENSION)) {
					lstElemMutex = var.getDimensions();
				}
				if (mutex.toString().equals(ConstantesXml.TYPE)) {
					lstElemMutex = new ArrayList<ElementStructure>();
					lstElemMutex.add(new Type(var.getType()));
				}

				if ((lstElemErreur = mutex.trouver(lstElemMutex, this.elementBlocLevel)).isEmpty() == false ) {
					lstErreur.add(creerErreurEnglobante(lstElemErreur,var));
				}
			}
		}
		return lstErreur;
	}

	/**
	 * 
	 * 
	 */
	private ArrayList<String []> trouverAttribut(CoVariable var) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();
		
		if (this.getAttributs() != null && this.getAttributs().isEmpty() == false) {

			Iterator iterAtt = this.getAttributs().iterator();
			while (iterAtt.hasNext()) {
				CoAttribute att = (CoAttribute) iterAtt.next();
				ArrayList elemErreur = null;
				if ((elemErreur = att.trouver(var.getAttribute(), this.elementBlocLevel)).isEmpty() == false) {
					lstErreur.add(creerErreurEnglobante(elemErreur,var));
				}
			}
		}
		return lstErreur;
	}

	/**
	 * 
	 * 
	 */
	public ArrayList<String []> trouverNbDimension(CoVariable variable) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();
		Integer rulesDimensionCount = 0;
		Integer netdcfDimensionCount = 0;  

		if (dimensionCount != null) {
			
			if (variable.getDimensions() != null) {
				rulesDimensionCount = dimensionCount.intValue();
				netdcfDimensionCount = variable.getDimensions().size();
				
				// Si pas le meme nombre de dimension
				if (rulesDimensionCount != netdcfDimensionCount) {
					String [] tab = {
							String.valueOf(ErreurRegles.NB_DIM_VAR),
							variable.getName(),
							netdcfDimensionCount.toString(),
							rulesDimensionCount.toString()};
					lstErreur.add(tab);
				}
			}
			// Si la variable n'a aucune dimension
			else {
				String [] tab = {
						String.valueOf(ErreurRegles.NB_DIM_VAR),
						variable.getName(),
						netdcfDimensionCount.toString(),
						rulesDimensionCount.toString()};
				lstErreur.add(tab);
			}
		}

		return lstErreur;
	}

	/**
	 * 
	 * 
	 */
	public ArrayList<String []> trouverType(CoVariable var) {
		ArrayList<String []> lstErreur = new ArrayList<String []>();

		if (type != null) {
			if (var.getType() != null) {
				if ((var.getType()).equals(type) == false) {
					// Erreur pas meme type
					String [] tab = {
							String.valueOf(ErreurRegles.TYPE_VAR),
							type,
							var.getName()};
					lstErreur.add(tab);
				}
			}
			else {
				// Erreur pas meme type
				String [] tab = {
						String.valueOf(ErreurRegles.TYPE_VAR),
						type,
						var.getName()};
				lstErreur.add(tab);
			}
		}
		return lstErreur;
	}
	
	/**
	 * 
	 * 
	 */
	public String [] creerErreurEnglobante(ArrayList lstElemErreur, ElementStructure elemEnglobant) {
		int codeErreur = 0;
		String [] temp = null;
		Iterator iterErreur = lstElemErreur.iterator();
		while (iterErreur.hasNext()) {
			String [] tab = (String []) iterErreur.next();

			switch (codeErreur = Integer.parseInt(tab[0])) {
				case ErreurRegles.DIM_MUTEX :
					codeErreur = ErreurRegles.DIM_MUTEX_VAR;
					break;
				case ErreurRegles.TYPE_MUTEX :
					codeErreur = ErreurRegles.TYPE_MUTEX_VAR;
					break;
				case ErreurRegles.DIM_NOT_FOUND :
					codeErreur = ErreurRegles.DIM_VAR;
					break;
				case ErreurRegles.TYPE_NOT_FOUND :
					codeErreur = ErreurRegles.TYPE_VAR;
					break;
				case ErreurRegles.ATT_VALUE :
					codeErreur = ErreurRegles.ATT_VAR_VALUE;
					break;
				case ErreurRegles.ATT_NOT_FOUND :
					codeErreur = ErreurRegles.ATT_VAR;
					break;
				case ErreurRegles.DIM_MUTEX_NOT_FOUND :
					codeErreur = ErreurRegles.DIM_MUTEX_VAR_NOT_FOUND;
					break;
				default :
					break;

			}
			// Recopie des autres informations
			temp = new String[tab.length + 1] ;
	    	temp[0] = String.valueOf(codeErreur);
	    	for (int i = 1; i < tab.length; i++) {
	    		temp[i] = tab[i];
	    	}
	    	temp[tab.length] = elemEnglobant.getName();
		}
		return temp;
	}

}
