package oco.structure;

import java.util.ArrayList;
import java.util.Iterator;

import oco.erreur.ErreurRegles;
import oco.traitement.ConstantesXml;

public class Mutex extends ElementCompose {
	
	// contient une liste coherente de type
	ArrayList<ElementStructure> donnees = null;
	
	String typeElem = null;

	
	/**
	 * Constructeur
	 * 
	 */
	public Mutex(String pType) {
		super();
		typeElem = pType;
	}
	
	/**
	 * Accesseur
	 * 
	 */
	public String toString() {
		return typeElem;
	}

	/**
	 * Ajouter un element du fichier de regles
	 * 
	 */
	public void ajouterElement(ElementStructure elem) {
		if (this.donnees == null) {
			this.donnees = new ArrayList<ElementStructure>();
		}
		this.donnees.add(elem);
	}

	/**
	 * 
	 * 
	 */
	public ArrayList trouver(ArrayList lst, String ruleLevel) {
		ArrayList<ElementStructure> lstNom = new ArrayList<ElementStructure>();
		ArrayList<ElementStructure> lstMutex = new ArrayList<ElementStructure>();
		ArrayList<String []> lstErreur = new ArrayList<String []>();
		
		// pour chaque element mutex
		Iterator iterRegles = this.donnees.iterator();
		while (iterRegles.hasNext()) {

			ElementStructure elemRegle = (ElementStructure) iterRegles.next();

			// stock le nom de l'element pour le message 
			if (elemRegle.getName() != null) {
				lstNom.add(elemRegle);
			}
			
			// mutex sur variable
			if (elemRegle.isComposed()) {
				CoVariable var = (CoVariable) elemRegle;
				if (var.variableExiste(lst) ) {
					lstMutex.add(var);
				}
			} else {
				if (elemRegle.trouver(lst, ruleLevel).isEmpty() == true) {
					lstMutex.add(elemRegle);
				}
			}
		}
		
		// plusieurs mutex trouve
		if (lstMutex.size() > 1) {
			mutexPresent(lstMutex, lstErreur);
		}
		// aucun mutex trouve
		if (lstMutex.isEmpty() == true) {
			int codeErreur = 0;
			if (typeElem.equals(ConstantesXml.DIMENSION)) {
				codeErreur = ErreurRegles.DIM_MUTEX_NOT_FOUND;
			}
			if (typeElem.equals(ConstantesXml.VARIABLE)) {
				codeErreur = ErreurRegles.VAR_MUTEX_NOT_FOUND;
			}
			if (typeElem.equals(ConstantesXml.TYPE)) {
				codeErreur = ErreurRegles.TYPE_MUTEX_VAR_NOT_FOUND;
			}
			String [] tab = {String.valueOf(codeErreur),recupElemList(lstNom)};
			lstErreur.add(tab);
		}
		// un seul mutex trouve
		if (lstMutex.size() == 1) {
			Object elemMutex = lstMutex.get(0);
			// verification des proprietes de la variable
			if (elemMutex instanceof CoVariable) {
				lstErreur.addAll(((CoVariable)elemMutex).verifierVariable());
			}
		}
		
		return lstErreur;
	}
	
	/**
	 * 
	 * 
	 */
	private void mutexPresent(ArrayList<ElementStructure> elem, ArrayList<String []> lstErreur) {
		int codeErreur = 0;
		if (typeElem.equals(ConstantesXml.DIMENSION)) {
			codeErreur = ErreurRegles.DIM_MUTEX;
		}
		if (typeElem.equals(ConstantesXml.VARIABLE)) {
			codeErreur = ErreurRegles.VAR_MUTEX;
		}
		if (typeElem.equals(ConstantesXml.TYPE)) {
			codeErreur = ErreurRegles.TYPE_MUTEX;
		}
		String [] tab = { String.valueOf(codeErreur), recupElemList(elem)};

		lstErreur.add(tab);
	}

	/**
	 * 
	 * 
	 */
	private String recupElemList(ArrayList lst) {

		StringBuffer lstNom = new StringBuffer();
		Iterator iter = lst.iterator();
		while (iter.hasNext()) {
			ElementStructure elem = (ElementStructure)iter.next();
			if (elem.getName() != null) {
				lstNom.append(elem.getName());
			}
			if (iter.hasNext()) {
				lstNom.append(",");
			}
		}
		return lstNom.toString();
	}
}
