package oco.traitement;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oco.FormatControl;
import oco.erreur.RessourceException;
import oco.structure.CoAttribute;
import oco.structure.CoDimension;
import oco.structure.CoVariable;

import ucar.ma2.ArrayChar;
import ucar.nc2.Attribute;
import ucar.nc2.Dimension;
import ucar.nc2.NetcdfFile;
import ucar.nc2.Variable;

/**
 *
 * Classe responsable du chargement du fichier netcdf
 *
 */
public class CoNetcdfFile {

	protected NetcdfFile ucarFile = null;
	ArrayList<CoDimension> dimensionList = null;
	ArrayList<CoAttribute> globalAttributeList = null;
	ArrayList<CoVariable> variableList = null;
	
	/**
	 * Constructeur
	 * 
	 */
	public CoNetcdfFile(NetcdfFile p_file) {
		super();
		this.ucarFile = p_file;
		this.dimensionList = new ArrayList<CoDimension>();
		this.globalAttributeList = new ArrayList<CoAttribute>();
		this.variableList = new ArrayList<CoVariable>();
	}

	/**
	 * 
	 * 
	 */
	public List retournerDimensions() {
		
		if (this.dimensionList.isEmpty()) {
			
			List list = this.ucarFile.getDimensions();
			
			Iterator iter = list.iterator();
			while (iter.hasNext()) {
				Dimension ucarDimension = (Dimension) iter.next();
				CoDimension coDimension = new CoDimension();
				coDimension.setName(ucarDimension.getName());
				if (ucarDimension.isUnlimited() == true) {
					coDimension.setValue("0");
				}
				else {
					coDimension.setValue(new Integer(ucarDimension.getLength()).toString());
				}
				this.dimensionList.add(coDimension);
			}
		}
		return this.dimensionList;
	}

	/**
	 * 
	 * 
	 */
	public List retournerGlobalAttributes() {

		if (this.globalAttributeList.isEmpty()) {
			
			List list = ucarFile.getGlobalAttributes();
			
			Iterator iter = list.iterator();
			while (iter.hasNext()) {
				Attribute ucarAttribute = (Attribute) iter.next();
				CoAttribute coGlobalAttribute = new CoAttribute();
				coGlobalAttribute.setName(ucarAttribute.getName());
				coGlobalAttribute.setValue(ucarAttribute.getStringValue());
				this.globalAttributeList.add(coGlobalAttribute);
			}
		}
		return this.globalAttributeList;
	}

	/**
	 * 
	 * 
	 */
	public List retournerVariables() {
		String valueAsString;
		Number valueAsNumber;
		
		if (this.variableList.isEmpty()) {
			
			List list = this.ucarFile.getVariables();
			
			Iterator iter = list.iterator();
			while (iter.hasNext()) {
				
				Variable ucarVariable = (Variable) iter.next();
				CoVariable coVariable = new CoVariable();
				coVariable.setName(ucarVariable.getName());
				coVariable.setType(ucarVariable.getDataType().toString());
				
				// chargement des attributs associes
				List dimList = ucarVariable.getAttributes();
				Iterator attIter = dimList.iterator();
				while (attIter.hasNext()) {
					Attribute ucarAttribute = (Attribute) attIter.next();
					CoAttribute coAttribute = new CoAttribute();
					valueAsNumber = ucarAttribute.getNumericValue();
					valueAsString = ucarAttribute.getStringValue();
					if (valueAsNumber != null) {
						coAttribute.setValue(valueAsNumber.toString());
					} else {
						coAttribute.setValue(valueAsString);
					}
					coAttribute.setName(ucarAttribute.getName());
					coVariable.getAttribute().add(coAttribute);
				}
				
				// chargement des dimensions associees
				dimList = ucarVariable.getDimensions();
				Iterator dimIter = dimList.iterator();
				while (dimIter.hasNext()) {
					Dimension ucarDimension = (Dimension) dimIter.next();
					CoDimension coDimension = new CoDimension();
					coDimension.setName(ucarDimension.getName());
					coVariable.getDimensions().add(coDimension);
				}
				this.variableList.add(coVariable);
			}
		}
		return this.variableList;
	}

	/**
	 * 
	 * 
	 */
	public String retournerValeurVariableChar(String p_name) throws RessourceException {
		
		Variable ucarVariable = this.ucarFile.findVariable(p_name);
        ArrayChar dataArray = null;
		try {
			dataArray = (ArrayChar) ucarVariable.read();
		} catch (IOException e) {
			FormatControl.report.addAppError(ConstantesProperties.error13);
			throw new RessourceException();
		}
        return (dataArray == null ? null : dataArray.getString());
	}
}
