package oco.traitement;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Properties;

import oco.FormatControl;
import oco.erreur.RessourceException;

public class OperationFichier {

	
	/**
	 * 
	 * 
	 */
	public static InputStream chargerFichier(String fileName) throws RessourceException {
		InputStream is = null;

		if (fileName == null) {
			FormatControl.report.addAppError(ConstantesProperties.error11, fileName);
			throw new RessourceException();
		}
		
		File f = null;
        try {
			f = new File(fileName);
			if (f.exists()) {
				is = new FileInputStream(f);
			}
			else {
				is = Thread.currentThread().getContextClassLoader().getResourceAsStream(fileName);
			}
		 } catch (FileNotFoundException e) {
			FormatControl.report.addAppError(ConstantesProperties.error11, fileName);
	     	throw new RessourceException();
	     }

		 if (is == null) {
			 FormatControl.report.addAppError(ConstantesProperties.error11, fileName);
			 throw new RessourceException();
		 }
		 return is;
	}

	/**
	 * 
	 * 
	 */
	public static Properties chargerFichierProperties(String fileName) throws RessourceException {
        Properties properties = new Properties();
        InputStream is = chargerFichier(fileName);

        try {
            properties.load(is);
        } catch (IOException e) {
			FormatControl.report.addAppError(ConstantesProperties.error11, fileName);
        	throw new RessourceException();
        }

        return properties;
	}

	/**
	 * 
	 * 
	 */
	public static ArrayList<File> recupererDesFichiers(String directory, String extension) throws RessourceException {

		ArrayList<File> retour = new ArrayList<File>();
		if (directory == null) {
			FormatControl.report.addAppError(ConstantesProperties.error12, directory);
			throw new RessourceException();
		}

		File fileDirectory = new File(directory);
		if (fileDirectory.exists()) {
			File [] lst = fileDirectory.listFiles();
			for (int i=0; i < lst.length; i++) {
				if (lst[i].isFile() && lst[i].getName().endsWith(extension) ) {
					retour.add(lst[i]);
				}
			}
		}
		else {
			FormatControl.report.addAppError(ConstantesProperties.error12, directory);
			throw new RessourceException();
		}
		return retour;
	}
	
	/**
	 * 
	 * 
	 */
	public static File recupererUnFichier(String directory, String fileName) throws RessourceException {
		File file = null;

		if (directory == null) {
			FormatControl.report.addAppError(ConstantesProperties.error12, directory);
			throw new RessourceException();
		}
		
		File fileDirectory = new File(directory);
		if (fileDirectory.exists()) {
			File [] lst = fileDirectory.listFiles();
			for (int i=0; i < lst.length; i++) {
				if (lst[i].isFile() && lst[i].getName().equals(fileName)) {
					file = lst[i];
				}
			}
		}
		else {
			FormatControl.report.addAppError(ConstantesProperties.error12, directory);
			throw new RessourceException();
		}
		return file;
	}
}
