package oco.traitement;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import oco.FormatControl;
import oco.erreur.InvalidXmlException;
import oco.erreur.MessageErrorManager;
import oco.erreur.OutMessage;
import oco.erreur.RessourceException;
import oco.erreur.XsdError;
import oco.structure.CoAttribute;
import oco.structure.CoVariable;
import oco.structure.ElementStructure;
import oco.structure.FactoryElementStructurel;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


public class XmlManager {
	Document doc = null;
	File rulesFile = null;
	MessageErrorManager report = null;

	
	/**
	 * Constructeur
	 * 
	 */
	public XmlManager(File xmlFile, MessageErrorManager errorManager) 
		throws RessourceException, InvalidXmlException {
		
		this.rulesFile = xmlFile;
		this.report = errorManager;

		DocumentBuilder documentBuilder = null;
		System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		
		try {
			documentBuilder = documentBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			// Parser configuration error
			FormatControl.report.addRuleError(ConstantesProperties.error21, this.rulesFile.getName());
			throw new RessourceException();
		}
		
		try {
			doc = documentBuilder.parse(this.rulesFile);
		} catch (SAXParseException e) {
			String message = this.rulesFile.getName();
			message += " : " + "line " + e.getLineNumber() + " column " + e.getColumnNumber();
			message += " : " + e.getMessage();
			// Can't load the rules file : 
			FormatControl.report.addRuleError(ConstantesProperties.error22, message);
			throw new InvalidXmlException();
		} catch (SAXException e) {
			// SAX error
			FormatControl.report.addRuleError(ConstantesProperties.error23, this.rulesFile.getName());
			throw new RessourceException();
		} catch (IOException e) {
			// Can't read the rules file : 
			FormatControl.report.addRuleError(ConstantesProperties.error24, this.rulesFile.getName());
			throw new RessourceException();
		}
	}
	
	/**
	 * Constructeur
	 * 
	 */
	public XmlManager(String xmlFileName, MessageErrorManager errorManager) 
		throws RessourceException, InvalidXmlException {
		
		this(new File(xmlFileName), errorManager);
	}

	/**
	 * Le fichier de regles est il valide par rapport au xsd ?
	 * 
	 */
	public void verifierFichierRegle(File xsdFile) throws RessourceException, InvalidXmlException {
		XMLReader reader = null;

		try {
		    reader = XMLReaderFactory.createXMLReader();
		} catch (SAXException e) {
			throw new RessourceException();
		}

		try {
		    reader.setFeature("http://xml.org/sax/features/validation", true);
		} catch (SAXNotRecognizedException e) {
			throw new RessourceException();
		} catch (SAXNotSupportedException e) {
			throw new RessourceException();
		}
        try {
            reader.setFeature("http://apache.org/xml/features/validation/schema", true);
        } catch (SAXNotRecognizedException e) {
			throw new RessourceException();
        } catch (SAXNotSupportedException e) {
			throw new RessourceException();
        }
        
        String id = "http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation";
        Object value = xsdFile.getAbsolutePath();
        try {
            reader.setProperty(id, value);
        } catch (SAXNotRecognizedException e) {
			throw new RessourceException();
        } catch (SAXNotSupportedException e) {
			throw new RessourceException();
        }

        XsdError handler = new XsdError();
        reader.setErrorHandler(handler);
        InputSource source = null;
        try {
            source = new InputSource(new FileInputStream(rulesFile));
        } catch (FileNotFoundException e) {
        	// Can't load the rules file : 
        	this.rulesFile.getName();
			FormatControl.report.addRuleError(ConstantesProperties.error22, this.rulesFile.getName());
			throw new RessourceException();
        }
        try {
            reader.parse(source);
        } catch (IOException e) {
        	// Can't read the rules file
			FormatControl.report.addRuleError(ConstantesProperties.error24, this.rulesFile.getName());
			throw new RessourceException();
        } catch (SAXException e) {
        	// SAX error
			FormatControl.report.addRuleError(ConstantesProperties.error23, this.rulesFile.getName());
			throw new RessourceException();
        }
        if (handler.hasErrorMessage() == true) {
        	String [] messages = handler.getMessage().split("\n");
        	String message;
			for (int i = 0; i < messages.length; i++) {
				message = messages[i];
				message = this.rulesFile.getName() + " : " + xsdFile.getName() + " : " + message;
				// The rules file is not valid :
				report.addRuleError(ConstantesProperties.error25, message);
			}
        	throw new InvalidXmlException();
        }
	}
	
	/**
	 * Le fichier de regles est il applicable ?
	 * 
	 */
	public void verifierApplicabilite(CoNetcdfFile netdcfFile) throws RessourceException {
		String expression = ConstantesXml.XPATH_QUERY;
		NodeList conditionApplication = null;
		CoAttribute attribut = null;
		CoVariable variable = null;
		String valeurRegle = null;
		String valeurNetcdf = null;
		String tag = null;
		String value = null;
		
		try {
			// Cration du XPath
			XPathFactory fabrique = XPathFactory.newInstance();
			XPath xpath = fabrique.newXPath();

			// Rcuperer le noeud APPLICABLE_IF
			XPathExpression exp = xpath.compile(expression);
			conditionApplication = (NodeList) exp.evaluate(doc, XPathConstants.NODESET);
			Node noeudApplicableIf = conditionApplication.item(0);

			// Si des conditions sont presentes
			if (noeudApplicableIf.hasChildNodes()) {
				NodeList listeConditions = noeudApplicableIf.getChildNodes();
				
				// Pour chaque condition
				for (int i = 0; i < listeConditions.getLength(); i++) {
					if (listeConditions.item(i).getNodeType() == Node.ELEMENT_NODE) {
						
						// Construire l'elememt correspondant  la condition
						Node noeudCondition = listeConditions.item(i);
						ElementStructure elemStruct = FactoryElementStructurel.createElementStructurel(noeudCondition);
						
						// Si la condition porte sur un attribut
						if (elemStruct.toString().equals(ConstantesXml.ATTRIBUTE)) {
							// Si le fichier netcdf ne contient pas l'attribut
							// ou la valeur de l'attribut ne correspond pas
							if (elemStruct.trouver((ArrayList)netdcfFile.retournerGlobalAttributes(), ConstantesXml.MANDATORY).isEmpty() == false)
								throw new RessourceException();
						}
						
						// Si la condition porte sur une variable
						if (elemStruct.toString().equals(ConstantesXml.VARIABLE)) {
							variable = FactoryElementStructurel.createSpecialVariable(noeudCondition);
							// Si le fichier netcdf ne contient pas la variable
							if (variable.trouver((ArrayList)netdcfFile.retournerVariables(), ConstantesXml.MANDATORY).isEmpty() == false) {
								throw new RessourceException();
							}
							// Si la valeur de la variable ne correspond pas
							valeurRegle = variable.getCharValue();
							valeurNetcdf = netdcfFile.retournerValeurVariableChar(variable.getName());
							if (valeurRegle.equals(valeurNetcdf) == false) {
								throw new RessourceException();
							}
						}
					}
				}
				
				// Ajout des informations du fichier de regle
				report.addValue(OutMessage.rulesFile, rulesFile.getName());
				// Pour chaque condition
				for (int i = 0; i < listeConditions.getLength(); i++) {
					if (listeConditions.item(i).getNodeType() == Node.ELEMENT_NODE) {
						
						// Construire l'elememt correspondant  la condition
						Node noeudCondition = listeConditions.item(i);
						ElementStructure elemStruct = FactoryElementStructurel.createElementStructurel(noeudCondition);
						
						// Si la condition porte sur un attribut
						if (elemStruct.toString().equals(ConstantesXml.ATTRIBUTE)) {
							attribut = FactoryElementStructurel.recupererAttribut(noeudCondition);
							tag = attribut.getName().toLowerCase();
							if (attribut.getValue() != null){
								value = attribut.getValue().trim();
							}
							else {
								if (attribut.getPattern() != null){
									value = attribut.getPattern().trim();
								}
							}
							report.addValue(tag, value);
						}
						
						// Si la condition porte sur une variable
						if (elemStruct.toString().equals(ConstantesXml.VARIABLE)) {
							variable = FactoryElementStructurel.createSpecialVariable(noeudCondition);
							valeurRegle = variable.getCharValue();
							tag = variable.getName().toLowerCase();
							value = variable.getCharValue().trim();
							report.addValue(tag, value);
						}
					}
				}
			}
		} catch (XPathExpressionException xpee) {
			xpee.printStackTrace();
		}
	}
	
	/**
	 * 
	 * 
	 */
	public Document getDoc() {
		return doc;
	}
}
